@@ -848,6 +848,16 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	g_free (launcher_location);
 }
 
+static void
+ensure_item_localefiled (GnomeDesktopItem *ditem, const char *field)
+{
+	if (gnome_desktop_item_get_string (ditem, field) == NULL) {
+		const char *localized = gnome_desktop_item_get_localestring (ditem, field);
+		if (localized != NULL)
+			gnome_desktop_item_set_string (ditem, field, localized);
+	}
+}
+
 static void
 really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 {
@@ -861,6 +871,15 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 
 		ditem = gnome_ditem_edit_get_ditem (dedit);
 
+		/* Make sure we set the "C" locale strings to the terms we set
+		   here.  This is so that if the user logs into another locale
+		   they get their own description there rather then empty.  It
+		   is not the C locale however, but the user created this entry
+		   herself so it's OK */
+		ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_NAME);
+		ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_GENERIC_NAME);
+		ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_COMMENT);
+
 		panel_launcher_save_ditem (ditem, gtk_window_get_screen (GTK_WINDOW (dialog)));
 		location = gnome_desktop_item_get_location (ditem);
 
@@ -892,7 +911,7 @@ ask_about_launcher (const char  *file,
 
 	dialog = gtk_dialog_new_with_buttons (
 				_("Create Launcher"),
-				GTK_WINDOW (panel->toplevel),
+				NULL,
 				0 /* flags */,
 				GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -935,6 +954,8 @@ ask_about_launcher (const char  *file,
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
+	gtk_window_set_screen (GTK_WINDOW (dialog), gtk_widget_get_screen (GTK_WIDGET (panel)));
+
 	gtk_widget_show_all (dialog);
 
 	gnome_ditem_edit_grab_focus (dee);
