@@ -21,11 +21,8 @@
 
 #include <libgnome/libgnome.h>
 
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "menu-fentry.h"
 #include "quick-desktop-reader.h"
