@@ -30,20 +30,32 @@
 #include <glade/glade-xml.h>
 #include <libgnomeui/gnome-color-picker.h>
 #include <libgnomeui/gnome-file-entry.h>
+#include <libgnomeui/gnome-icon-entry.h>
 
 #include "panel-profile.h"
 #include "panel-gconf.h"
 #include "panel-util.h"
+#include "panel-globals.h"
 
 typedef struct {
 	PanelToplevel *toplevel;
 
 	GtkWidget     *properties_dialog;
 
+	GtkWidget     *general_table;
+	GtkWidget     *general_vbox;
 	GtkWidget     *name_entry;
+	GtkWidget     *name_label;
 	GtkWidget     *orientation_menu;
+	GtkWidget     *orientation_label;
+	GtkWidget     *size_widgets;
 	GtkWidget     *size_spin;
-	GtkWidget     *expand_toggle;
+	GtkWidget     *size_label;
+	GtkWidget     *size_label_pixels;
+	GtkWidget     *icon_align;
+	GtkWidget     *icon_entry;
+	GtkWidget     *icon_label;
+  	GtkWidget     *expand_toggle;
 	GtkWidget     *autohide_toggle;
 	GtkWidget     *hidebuttons_toggle;
 	GtkWidget     *arrows_toggle;
@@ -53,11 +65,20 @@ typedef struct {
 	GtkWidget     *color_widgets;
 	GtkWidget     *image_widgets;
 	GtkWidget     *color_picker;
+	GtkWidget     *color_label;
 	GtkWidget     *image_entry;
 	GtkWidget     *opacity_scale;
+	GtkWidget     *opacity_label;
+	GtkWidget     *opacity_legend;
+
+	GtkWidget     *writability_warn_general;
+	GtkWidget     *writability_warn_background;
 
 	guint          toplevel_notify;
 	guint          background_notify;
+
+	/* The theme directory of the icon, see bug #119209 */
+	char          *icon_theme_dir;
 } PanelPropertiesDialog;
 
 static GQuark panel_properties_dialog_quark = 0;
@@ -81,17 +102,24 @@ panel_properties_dialog_free (PanelPropertiesDialog *dialog)
 		gtk_widget_destroy (dialog->properties_dialog);
 	dialog->properties_dialog = NULL;
 
+	g_free (dialog->icon_theme_dir);
+	dialog->icon_theme_dir = NULL;
+
 	g_free (dialog);
 }
 
 static char *
-get_name (PanelToplevel *toplevel)
+get_name (PanelToplevel *toplevel,
+	  gboolean       is_attached)
 {
 	char *name;
 
-	name = panel_profile_get_toplevel_name (toplevel);
-	if (!name)
-		name = g_strdup (panel_toplevel_get_description (toplevel));
+	if (!is_attached) {
+		name = panel_profile_get_toplevel_name (toplevel);
+		if (!name)
+			name = g_strdup (panel_toplevel_get_description (toplevel));
+	} else
+		name = panel_profile_get_attached_tooltip (toplevel);
 
 	return name;
 }
@@ -100,23 +128,47 @@ static void
 panel_properties_dialog_name_changed (PanelPropertiesDialog *dialog,
 				      GtkEntry              *entry)
 {
-	panel_profile_set_toplevel_name (dialog->toplevel, gtk_entry_get_text (entry));
+	const char *name;
+
+	name = gtk_entry_get_text (entry);
+
+	if (!panel_toplevel_get_is_attached (dialog->toplevel))
+		panel_profile_set_toplevel_name (dialog->toplevel, name);
+	else
+		panel_profile_set_attached_tooltip (dialog->toplevel, name);
 }
 
 static void
 panel_properties_dialog_setup_name_entry (PanelPropertiesDialog *dialog,
 					  GladeXML              *gui)
 {
-	char *name;
+	char     *name;
+	gboolean  is_attached;
+	gboolean  is_writable;
 
 	dialog->name_entry = glade_xml_get_widget (gui, "name_entry");
+	g_return_if_fail (dialog->name_entry != NULL);
 
-	name = get_name (dialog->toplevel);
-	gtk_entry_set_text (GTK_ENTRY (dialog->name_entry), name);
+	dialog->name_label = glade_xml_get_widget (gui, "name_label");
+	g_return_if_fail (dialog->name_label != NULL);
+
+	is_attached = panel_toplevel_get_is_attached (dialog->toplevel);
+
+	name = get_name (dialog->toplevel, is_attached);
+	gtk_entry_set_text (GTK_ENTRY (dialog->name_entry), name ? name : "");
 	g_free (name);
 
 	g_signal_connect_swapped (dialog->name_entry, "changed",
 				  G_CALLBACK (panel_properties_dialog_name_changed), dialog);
+
+	is_writable = !is_attached ? panel_profile_is_writable_toplevel_name    (dialog->toplevel) :
+	                             panel_profile_is_writable_attached_tooltip (dialog->toplevel);
+
+	if (!is_writable) {
+		gtk_widget_set_sensitive (dialog->name_entry, FALSE);
+		gtk_widget_set_sensitive (dialog->name_label, FALSE);
+		gtk_widget_show (dialog->writability_warn_general);
+	}
 }
 
 /* Sucky: menu order is Top, Bottom, Left, Right */
@@ -184,6 +236,9 @@ panel_properties_dialog_setup_orientation_menu (PanelPropertiesDialog *dialog,
 	PanelOrientation orientation;
 
 	dialog->orientation_menu = glade_xml_get_widget (gui, "orientation_menu");
+	g_return_if_fail (dialog->orientation_menu != NULL);
+	dialog->orientation_label = glade_xml_get_widget (gui, "orientation_label");
+	g_return_if_fail (dialog->orientation_label != NULL);
 
 	orientation = panel_profile_get_toplevel_orientation (dialog->toplevel);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (dialog->orientation_menu),
@@ -192,6 +247,12 @@ panel_properties_dialog_setup_orientation_menu (PanelPropertiesDialog *dialog,
 	g_signal_connect_swapped (dialog->orientation_menu, "changed",
 				  G_CALLBACK (panel_properties_dialog_orientation_changed),
 				  dialog);
+
+	if ( ! panel_profile_is_writable_toplevel_orientation (dialog->toplevel)) {
+		gtk_widget_set_sensitive (dialog->orientation_menu, FALSE);
+		gtk_widget_set_sensitive (dialog->orientation_label, FALSE);
+		gtk_widget_show (dialog->writability_warn_general);
+	}
 }
 
 static void
@@ -206,7 +267,14 @@ static void
 panel_properties_dialog_setup_size_spin (PanelPropertiesDialog *dialog,
 					 GladeXML              *gui)
 {
+	dialog->size_widgets = glade_xml_get_widget (gui, "size_widgets");
+	g_return_if_fail (dialog->size_widgets != NULL);
 	dialog->size_spin = glade_xml_get_widget (gui, "size_spin");
+	g_return_if_fail (dialog->size_spin != NULL);
+	dialog->size_label = glade_xml_get_widget (gui, "size_label");
+	g_return_if_fail (dialog->size_label != NULL);
+	dialog->size_label_pixels = glade_xml_get_widget (gui, "size_label_pixels");
+	g_return_if_fail (dialog->size_label_pixels != NULL);
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->size_spin),
 				   panel_profile_get_toplevel_size (dialog->toplevel));
@@ -214,8 +282,96 @@ panel_properties_dialog_setup_size_spin (PanelPropertiesDialog *dialog,
 	g_signal_connect_swapped (dialog->size_spin, "value_changed",
 				  G_CALLBACK (panel_properties_dialog_size_changed),
 				  dialog);
+
+	if ( ! panel_profile_is_writable_toplevel_size (dialog->toplevel)) {
+		gtk_widget_set_sensitive (dialog->size_spin, FALSE);
+		gtk_widget_set_sensitive (dialog->size_label, FALSE);
+		gtk_widget_set_sensitive (dialog->size_label_pixels, FALSE);
+		gtk_widget_show (dialog->writability_warn_general);
+	}
 }
 
+static void
+panel_properties_dialog_icon_changed (PanelPropertiesDialog *dialog,
+				      GnomeIconEntry        *entry)
+{
+	const char *icon = NULL;
+        char       *freeme = NULL;
+                                                                                                                                                              
+        icon = gnome_icon_entry_get_filename (entry);
+                                                                                                                                                              
+        /* Strip dir from the icon path if in the icon
+         * theme directory.  See bug #119209
+         */
+        if (icon && g_path_is_absolute (icon)) {
+                char *dir;
+                                                                                                                                                              
+                dir = g_path_get_dirname (icon);
+                                                                                                                                                              
+                if (dir && dialog->icon_theme_dir && strcmp (dir, dialog->icon_theme_dir) == 0)
+                        icon = freeme = g_path_get_basename (icon);
+                                                                                                                                                              
+                g_free (dir);
+        }
+                                                                                                                                                              
+        panel_profile_set_attached_custom_icon (dialog->toplevel, icon);
+                                                                                                                                                              
+        g_free (freeme);
+}
+
+static void
+panel_properties_dialog_setup_icon_entry (PanelPropertiesDialog *dialog,
+					  GladeXML              *gui)
+{
+	char *custom_icon;
+
+	dialog->icon_align = glade_xml_get_widget (gui, "icon_align");
+	g_return_if_fail (dialog->icon_align != NULL);
+
+	dialog->icon_entry = glade_xml_get_widget (gui, "icon_entry");
+	g_return_if_fail (dialog->icon_entry != NULL);
+
+	dialog->icon_label = glade_xml_get_widget (gui, "icon_label");
+	g_return_if_fail (dialog->icon_label != NULL);
+
+	dialog->icon_theme_dir = NULL;
+	custom_icon = panel_profile_get_attached_custom_icon (dialog->toplevel);
+	if (custom_icon != NULL &&
+	    ! g_path_is_absolute (custom_icon)) {
+		/* if the icon is not absolute path name it comes from the
+		   theme, and as such we wish to store the theme directory
+		   where it comes from.  See bug #119209 */
+		char *icon = gnome_desktop_item_find_icon (panel_icon_theme,
+							   custom_icon,
+							   48 /* FIXME: preffered_size */,
+							   0 /* flags */);
+		if (icon != NULL)
+			dialog->icon_theme_dir = g_path_get_dirname (icon);
+		g_free (custom_icon);
+		custom_icon = icon;
+	}
+	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dialog->icon_entry), custom_icon);
+	g_free (custom_icon);
+
+	if (dialog->icon_theme_dir == NULL) {
+		/* use the default pixmap directory as the standard icon_theme_dir,
+		 * since the standard directory is themed */
+		g_object_get (G_OBJECT (dialog->icon_entry), "pixmap_subdir",
+			      &(dialog->icon_theme_dir), NULL);
+	}
+
+	g_signal_connect_swapped (dialog->icon_entry, "changed",
+				  G_CALLBACK (panel_properties_dialog_icon_changed), dialog);
+
+	if (!panel_profile_is_writable_attached_custom_icon (dialog->toplevel)) {
+		gtk_widget_set_sensitive (dialog->icon_entry, FALSE);
+		gtk_widget_set_sensitive (dialog->icon_label, FALSE);
+		gtk_widget_show (dialog->writability_warn_general);
+	}
+}
+
+/* Note: this is only for toggle buttons on the general page, if needed for togglebuttons
+   elsewhere you must make this respect the writability warning thing for the right page */
 #define SETUP_TOGGLE_BUTTON(wid, n, p)                                                            \
 	static void                                                                               \
 	panel_properties_dialog_##n (PanelPropertiesDialog *dialog,                               \
@@ -233,6 +389,10 @@ panel_properties_dialog_setup_size_spin (PanelPropertiesDialog *dialog,
 					      panel_profile_get_toplevel_##p (dialog->toplevel)); \
 		g_signal_connect_swapped (dialog->n, "toggled",                                   \
 					  G_CALLBACK (panel_properties_dialog_##n), dialog);      \
+		if ( ! panel_profile_is_writable_toplevel_##p (dialog->toplevel)) {               \
+			gtk_widget_set_sensitive (dialog->n, FALSE);                              \
+			gtk_widget_show (dialog->writability_warn_general);			  \
+		}										  \
 	}
 
 SETUP_TOGGLE_BUTTON ("expand_toggle",      expand_toggle,      expand)
@@ -262,6 +422,9 @@ panel_properties_dialog_setup_color_picker (PanelPropertiesDialog *dialog,
 	PanelColor color;
 
 	dialog->color_picker = glade_xml_get_widget (gui, "color_picker");
+	g_return_if_fail (dialog->color_picker != NULL);
+	dialog->color_label = glade_xml_get_widget (gui, "color_label");
+	g_return_if_fail (dialog->color_label != NULL);
 
 	panel_profile_get_background_color (dialog->toplevel, &color);
 
@@ -274,6 +437,12 @@ panel_properties_dialog_setup_color_picker (PanelPropertiesDialog *dialog,
 	g_signal_connect_swapped (dialog->color_picker, "color_set",
 				  G_CALLBACK (panel_properties_dialog_color_changed),
 				  dialog);
+
+	if ( ! panel_profile_is_writable_background_color (dialog->toplevel)) {
+		gtk_widget_set_sensitive (dialog->color_picker, FALSE);
+		gtk_widget_set_sensitive (dialog->color_label, FALSE);
+		gtk_widget_show (dialog->writability_warn_background);
+	}
 }
 
 static void
@@ -308,6 +477,11 @@ panel_properties_dialog_setup_image_entry (PanelPropertiesDialog *dialog,
 	g_signal_connect_swapped (dialog->image_entry, "changed",
 				  G_CALLBACK (panel_properties_dialog_image_changed),
 				  dialog);
+
+	if ( ! panel_profile_is_writable_background_image (dialog->toplevel)) {
+		gtk_widget_set_sensitive (dialog->image_entry, FALSE);
+		gtk_widget_show (dialog->writability_warn_background);
+	}
 }
 
 static void
@@ -331,6 +505,11 @@ panel_properties_dialog_setup_opacity_scale (PanelPropertiesDialog *dialog,
 	gdouble percentage;
 
 	dialog->opacity_scale = glade_xml_get_widget (gui, "opacity_scale");
+	g_return_if_fail (dialog->opacity_scale != NULL);
+	dialog->opacity_label = glade_xml_get_widget (gui, "opacity_label");
+	g_return_if_fail (dialog->opacity_label != NULL);
+	dialog->opacity_legend = glade_xml_get_widget (gui, "opacity_legend");
+	g_return_if_fail (dialog->opacity_legend != NULL);
 
 	opacity = panel_profile_get_background_opacity (dialog->toplevel);
 
@@ -341,6 +520,13 @@ panel_properties_dialog_setup_opacity_scale (PanelPropertiesDialog *dialog,
 	g_signal_connect_swapped (dialog->opacity_scale, "value_changed",
 				  G_CALLBACK (panel_properties_dialog_opacity_changed),
 				  dialog);
+
+	if ( ! panel_profile_is_writable_background_opacity (dialog->toplevel)) {
+		gtk_widget_set_sensitive (dialog->opacity_scale, FALSE);
+		gtk_widget_set_sensitive (dialog->opacity_label, FALSE);
+		gtk_widget_set_sensitive (dialog->opacity_legend, FALSE);
+		gtk_widget_show (dialog->writability_warn_background);
+	}
 }
 
 static void
@@ -418,14 +604,24 @@ panel_properties_dialog_setup_background_radios (PanelPropertiesDialog *dialog,
 	g_signal_connect_swapped (dialog->image_radio, "toggled",
 				  G_CALLBACK (panel_properties_dialog_background_toggled),
 				  dialog);
+
+	if ( ! panel_profile_is_writable_background_type (dialog->toplevel)) {
+		gtk_widget_set_sensitive (dialog->default_radio, FALSE);
+		gtk_widget_set_sensitive (dialog->color_radio, FALSE);
+		gtk_widget_set_sensitive (dialog->image_radio, FALSE);
+		gtk_widget_show (dialog->writability_warn_background);
+	}
 }
 
 static void
 panel_properties_update_arrows_toggle_visible (PanelPropertiesDialog *dialog,
 					       GtkToggleButton       *toggle)
 {
-	gtk_widget_set_sensitive (dialog->arrows_toggle,
-				  gtk_toggle_button_get_active (toggle));
+	if (gtk_toggle_button_get_active (toggle))
+		gtk_widget_set_sensitive (dialog->arrows_toggle,
+					  panel_profile_is_writable_toplevel_enable_arrows (dialog->toplevel));
+	else
+		gtk_widget_set_sensitive (dialog->arrows_toggle, FALSE);
 }
 
 static void
@@ -435,7 +631,7 @@ panel_properties_dialog_response (PanelPropertiesDialog *dialog,
 {
 	switch (response) {
 	case GTK_RESPONSE_CLOSE:
-		gtk_widget_hide (properties_dialog);
+		gtk_widget_destroy (properties_dialog);
 		break;
 	case GTK_RESPONSE_HELP:
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (properties_dialog)),
@@ -446,6 +642,14 @@ panel_properties_dialog_response (PanelPropertiesDialog *dialog,
 	}
 }
 
+static void
+panel_properties_dialog_destroy (PanelPropertiesDialog *dialog)
+{
+	g_object_set_qdata (G_OBJECT (dialog->toplevel),
+			    panel_properties_dialog_quark,
+			    NULL);
+}
+
 static void
 panel_properties_dialog_update_name (PanelPropertiesDialog *dialog,
 				     GConfValue            *value)
@@ -633,13 +837,85 @@ panel_properties_dialog_background_notify (GConfClient           *client,
 		panel_properties_dialog_update_background_image (dialog, value);
 }
 
+static void
+panel_properties_dialog_remove_orientation_menu (PanelPropertiesDialog *dialog)
+{
+	GtkContainer *container = GTK_CONTAINER (dialog->general_table);
+	GtkTable     *table     = GTK_TABLE (dialog->general_table);
+
+	g_object_ref (dialog->size_label);
+	g_object_ref (dialog->size_widgets);
+	g_object_ref (dialog->icon_label);
+	g_object_ref (dialog->icon_align);
+
+	gtk_container_remove (container, dialog->orientation_label);
+	gtk_container_remove (container, dialog->orientation_menu);
+	gtk_container_remove (container, dialog->size_label);
+	gtk_container_remove (container, dialog->size_widgets);
+	gtk_container_remove (container, dialog->icon_label);
+	gtk_container_remove (container, dialog->icon_align);
+
+	gtk_table_attach_defaults (table, dialog->size_label,   0, 1, 1, 2);
+	gtk_table_attach_defaults (table, dialog->size_widgets, 1, 2, 1, 2);
+	gtk_table_attach_defaults (table, dialog->icon_label,   0, 1, 2, 3);
+	gtk_table_attach_defaults (table, dialog->icon_align,   1, 2, 2, 3);
+
+	dialog->orientation_label = NULL;
+	dialog->orientation_menu = NULL;
+	g_object_unref (dialog->size_label);
+	g_object_unref (dialog->size_widgets);
+	g_object_unref (dialog->icon_label);
+	g_object_unref (dialog->icon_align);
+
+	gtk_table_resize (table, 3, 2);
+}
+
+static void
+panel_properties_dialog_remove_icon_entry (PanelPropertiesDialog *dialog)
+{
+	GtkContainer *container = GTK_CONTAINER (dialog->general_table);
+
+	gtk_container_remove (container, dialog->icon_label);
+	gtk_container_remove (container, dialog->icon_align);
+
+	dialog->icon_label = NULL;
+	dialog->icon_align = NULL;
+	dialog->icon_entry = NULL;
+
+	gtk_table_resize (GTK_TABLE (dialog->general_table), 3, 2);
+}
+
+static void
+panel_properties_dialog_remove_toggles (PanelPropertiesDialog *dialog)
+{
+	GtkContainer *container = GTK_CONTAINER (dialog->general_vbox);
+
+	gtk_container_remove (container, dialog->autohide_toggle);
+	gtk_container_remove (container, dialog->expand_toggle);
+
+	dialog->autohide_toggle = NULL;
+	dialog->expand_toggle   = NULL;
+}
+
+static void
+panel_properties_dialog_update_for_attached (PanelPropertiesDialog *dialog,
+					     gboolean               attached)
+{
+	if (!attached)
+		panel_properties_dialog_remove_icon_entry (dialog);
+	else {
+		panel_properties_dialog_remove_toggles (dialog);
+		panel_properties_dialog_remove_orientation_menu (dialog);
+	}
+}
+
 static PanelPropertiesDialog *
 panel_properties_dialog_new (PanelToplevel *toplevel,
 			     GladeXML      *gui)
 {
 	PanelPropertiesDialog *dialog;
 
-	dialog = g_new (PanelPropertiesDialog, 1);
+	dialog = g_new0 (PanelPropertiesDialog, 1);
 
 	g_object_set_qdata_full (G_OBJECT (toplevel),
 				 panel_properties_dialog_quark,
@@ -651,13 +927,22 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 	dialog->properties_dialog = glade_xml_get_widget (gui, "panel_properties_dialog");
 	g_signal_connect_swapped (dialog->properties_dialog, "response",
 				  G_CALLBACK (panel_properties_dialog_response), dialog);
+	g_signal_connect_swapped (dialog->properties_dialog, "destroy",
+				  G_CALLBACK (panel_properties_dialog_destroy), dialog);
 
 	gtk_window_set_screen (GTK_WINDOW (dialog->properties_dialog),
 			       gtk_window_get_screen (GTK_WINDOW (toplevel)));
 
+	dialog->writability_warn_general = glade_xml_get_widget (gui, "writability_warn_general");
+	dialog->writability_warn_background = glade_xml_get_widget (gui, "writability_warn_background");
+
+	dialog->general_vbox  = glade_xml_get_widget (gui, "general_vbox");
+	dialog->general_table = glade_xml_get_widget (gui, "general_table");
+
 	panel_properties_dialog_setup_name_entry         (dialog, gui);
 	panel_properties_dialog_setup_orientation_menu   (dialog, gui);
 	panel_properties_dialog_setup_size_spin          (dialog, gui);
+	panel_properties_dialog_setup_icon_entry         (dialog, gui);
 	panel_properties_dialog_setup_expand_toggle      (dialog, gui);
 	panel_properties_dialog_setup_autohide_toggle    (dialog, gui);
 	panel_properties_dialog_setup_hidebuttons_toggle (dialog, gui);
@@ -688,6 +973,9 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 			(GConfClientNotifyFunc) panel_properties_dialog_background_notify,
 			dialog);
 
+	panel_properties_dialog_update_for_attached (dialog,
+						     panel_toplevel_get_is_attached (dialog->toplevel));
+
 	gtk_widget_show (dialog->properties_dialog);
 
 	return dialog;
